#
# News System v3 for eggdrop 1.1
# by Ernst <baschneo@trick.informatik.uni-stuttgart.de>	9/7/97
#
# Read the README file first, please.
#
# All configuration is done in news3.conf! You don't need to edit anything in
# this file (news3.tcl), unless you want to!

# - some useful functions -----------------------------------------------------

if {[info commands isinteger] == ""} {
	proc isinteger {arg} {
		if {[string length $arg] == 0} {return 0}
		set ctr 0
		while {$ctr < [string length $arg]} {
			if {![string match \[0-9\] [string index $arg $ctr]]} {return 0}
			set ctr [expr $ctr + 1]
		}
		return 1
	}
}

if {[info commands hostmask] != ""} {
	# Streak's maskhost procedure loaded, so use it (hostmask.tcl)
	proc newsmaskhost { ident } { hostmask $ident }
} {
	# Nope, so use eggdrops internal
	proc newsmaskhost { ident } { maskhost $ident }
}

# Removes a user key from xtra field
proc user-del {handle key} {
	set xtra [getxtra $handle]
	for {set i 0} {$i < [llength $xtra]} {incr i} {
		set this [lindex $xtra $i]
		if {[string compare [lindex $this 0] $key] == 0} {
			setxtra $handle [lreplace $xtra $i $i]
			return
		}
	}
	return
}

# - read the configuration file -----------------------------------------------

proc news { args } {
	global news
	set chan [string tolower [lindex $args 0]]
	if {[string index $chan 0] != "#"} { set chan "#$chan" }
	if {[lsearch [string tolower [channels]] $chan] == -1} {
		putlog "news config warning: channel $chan is not monitored by me"
		return 0
	}
	# Create a fresh configuration item for this channel with default values
	catch {unset news($chan)}
	lappend news($chan) "[string range $chan 1 end].news"
	lappend news($chan) "1"
	lappend news($chan) "0"
	lappend news($chan) "users"
	lappend news($chan) "0"
	lappend news($chan) "0"
	set config [lindex $args 1]
	# Parse configuration options, changing the defaults
	for {set i 0} {$i < [llength $config]} {incr i} {
		switch [string tolower [lindex $config $i]] {
		  "newsfile" {
			incr i
			if {![file exists [lindex $config $i]]} {
				set err [catch {set temp [open [lindex $config $i] a+]}]
				if {$err} {
					putlog "news $chan config warning: Could not write to [lindex $config $i] file. Using default: [lindex $news($chan) 0]"
					set err [catch {set temp [open [lindex $news($chan) 0] a+]}]
					if {$err} {
						putlog "news $chan config error: Something is real wrong with the filename [lindex $news($chan) 0]."
						break
					} {
						close $temp
					}
				} {
					close $temp
					set news($chan) "[lreplace $news($chan) 0 0 [lindex $config $i]]"
				}
			} {
				set news($chan) "[lreplace $news($chan) 0 0 [lindex $config $i]]"
			}
		  }
		  "openread" {
			incr i
			if {[lindex $config $i] != 0 && [lindex $config $i] != 1} {
				putlog "news $chan config warning: Only values 1 or 0 allowed for openread"
			} {
				set news($chan) "[lreplace $news($chan) 1 1 [lindex $config $i]]"
			}
		  }
		  "noticeonjoin" {
			incr i
			if {[lindex $config $i] != 0 && [lindex $config $i] != 1} {
				putlog "news $chan config warning: Only values 1 or 0 allowed for noticeonjoin"
			} {
				set news($chan) "[lreplace $news($chan) 2 2 [lindex $config $i]]"
			}
		  }
		  "allowwrite" {
			incr i
			set chosen [string tolower [lindex $config $i]]
			if {$chosen != "all" && $chosen != "users" && $chosen != "flag"} {
				putlog "news $chan config warning: Only values 'all', 'users' or 'flag X' allowed for allowwrite"
			} {
				if {$chosen == "flag"} {
					incr i
					set flag [lindex $config $i]
					set news($chan) [lreplace $news($chan) 3 3 "flag [lindex $config $i]"]
				} {
					set news($chan) "[lreplace $news($chan) 3 3 [lindex $config $i]]"
				}
			}
		  }
		  "expiredays" {
			incr i
			if {![isinteger [lindex $config $i]]} {
				putlog "news $chan config warning: expiredays must be integer but is [lindex $config $i]"
			} {
				set news($chan) "[lreplace $news($chan) 4 4 [lindex $config $i]]"
			}
		  }
		  "maxnews" {
			incr i
			if {![isinteger [lindex $config $i]]} {
				putlog "news $chan config warning: maxmsgs must be integer but is [lindex $config $i]"
			} {
				set news($chan) "[lreplace $news($chan) 5 5 [lindex $config $i]]"
			}
		  }
		  default {
			putlog "news $chan config warning: unknown news parameter: [lindex $config $i]"
		  }
		}
	}
}

set newsconf "[file dirname [info script]]/news3.conf"
if {![file exist $newsconf]} {
	putlog "news warning: No configuration: $newsconf not found"
} {
	# Wait 1 sec, for the dynamic channel configuration to be loaded
	utimer 1 "news_loadconfig $newsconf"
}
unset newsconf

proc news_loadconfig { file } {
	global news news_ncount news_newnews news_lastseen news_biggest_ts
	source $file
	# These are internal variables and shouldn't be changed here
	foreach chan [array names news] {
		set news_ncount($chan) -1
		set news_newnews($chan) 0
		set news_lastseen($chan) 0
		set news_biggest_ts($chan) 0
		news_count $chan
	}
	catch {unset chan}
	if {[array names news] != ""} {
		putlog "news configuration for [array names news] loaded"
	} {
		set news ""
	}
}

# - here comes the script -----------------------------------------------------

# Is there any news?
# EB 4/7: Multiple channels
proc news_thereisnews { chan } {
	global news
	if {[news_count $chan] == 0} {
		return 0
	} else {
		return 1
	}
}

# How many news items are there?  The first time this is called, it will
# read through the file and set the value of news_ncount.  In future, the
# value of the variable will be used, rather than reading the file each time.
# Also updates the news_biggest_ts
# EB 4/7: Multiple channels
proc news_count { chan } {
	global news news_ncount news_biggest_ts
	# only ever go through reading the file once -- the first time it's called
	if {$news_ncount($chan) == -1} {
		set news_ncount($chan) 0
		set fd [open [lindex $news($chan) 0] r]
		while {![eof $fd]} {
			set inp [gets $fd]
			if {[eof $fd]} {break}
			if {[string trim $inp " "] == ""} {continue}
			if {[lindex $inp 1] > $news_biggest_ts($chan)} {
				set news_biggest_ts($chan) [lindex $inp 1]
			}
			incr news_ncount($chan)
		}
		close $fd
	}
	return $news_ncount($chan)
}

# Internal procedure that, given a users handle, tells how many news items
# there are in the news file that they haven't read
# EB 4/7: Multiple channels
proc news_unread { hand chan } {
	global news
	set unread 0
	set user_ts [news_get_ts $hand $chan]
	set fd [open [lindex $news($chan) 0] r]
	while {![eof $fd]} {
		set inp [gets $fd]
		if {[eof $fd]} {break}
		if {[string trim $inp " "] == ""} {continue}
		if {[lindex $inp 1] > $user_ts} {
			incr unread
		}
	}
	close $fd
	return $unread
}

# Internal procedure to print out news from the file, since it's done more
# than once.  'item' should = 0 if all items	are to be printed, or a single
# number to print a specific item.  If the timestamp is non-zero, only news
# items more recent than this timestamp are printed.
# EB 4/7: Multiple channels, DCC/IRC in one proc
proc news_print { nick idx item timestamp chan } {
	global news
	set next 1
	set fd [open [lindex $news($chan) 0] r]
	while {![eof $fd]} {
		set inp [gets $fd]
		if {[eof $fd]} {break}
		if {[string trim $inp " "] == ""} {continue}
		# timestamp == 0 to show all items
		if {[lindex $inp 1] > $timestamp} {
			set who [lindex $inp 0]
			set date [string range [lrange [ctime [lindex $inp 1]] 1 3] 0 11]
			set msg [lrange $inp 2 end]
			if {$idx == 0} {
				putnotc $nick [format "%d. \[%s\] \(%s\) %s" $next $who $date $msg]
			} {
				putdcc $idx [format "%d. \[%s\] \(%s\) %s" $next $who $date $msg]
			}
		}
		incr next
	}
	close $fd
	return 1
}

# Get a users "last read" timestamp from the userfile
# Modified in 1.0 to get the timestamp from the 'xtra' field, not the comment
# EB 20/6: read using user-get proc from toolkit.tcl
# EB 4/7: New format: {news#portugal 8000800} (for multiple chans)
proc news_get_ts { hand chan } {
	set have_ts [user-get $hand "news$chan"]
	scan $have_ts "%d" have_ts
	if {$have_ts > 0} {
		return $have_ts
	} else {
		return 0
	}
}

# Set a users "last read" timestamp in the userfile
# Modified in 1.0 to store the timestamp in the 'xtra' field, not the comment
# EB 20/6: write using user-set proc from toolkit.tcl
# EB 4/7: New format: {news#portugal 8000800} (multiple chans)
proc news_set_ts {hand new_ts chan} {
	user-set $hand "news$chan" $new_ts
	return
}

# Add a news item
# EB 4/7: Multiple channels, DCC/IRC in one proc
proc news_add { nickidx hostdcc hand chan arg } {
# nickidx = irc nickname of poster or idx if it was via dcc
# hostdcc = uhost if from irc or "dcc" if via dcc
# hand = user handle which will show up in newsfile
	global news botnick news_ncount news_newnews news_biggest_ts
	global news_ignore_strangers
	set message [lrange $arg 1 end]
	if {$message == ""} {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "Usage: .news add \[#channel\] \"your message...\""
			putdcc $nickidx "       Adds \"your message...\" to the channel news."
		} {
			if {!$news_ignore_strangers || $hand != "*"} {
				putnotc $nickidx "Usage: /msg $botnick news add \[#channel\] \"your message...\""
				putnotc $nickidx "       Adds \"your message...\" to the channel news."
			}
		}
		return 0
	}
	# EB 9/7: To support any users to add news, add unknown users to userfile
	if {$hand == "*"} {
		set uhost [newsmaskhost $nickidx!$hostdcc]
		if {![adduser $nickidx $uhost]} {
			putnotc $nickidx "Could not add you as an user. Try to use another nickname."
			return 0
		} {
			setcomment $nickidx "[ctime [unixtime]] Added for news add"
			set hand $nickidx
		}
	}
	set fd [open [lindex $news($chan) 0] a]
	puts $fd [format "%s %s %s" $hand [unixtime] $message]
	close $fd
	# Do not just use 'incr' in case news_ncount currently = -1
	set news_ncount($chan) [expr [news_count $chan] + 1]
	set news_newnews($chan) 1
	set news_biggest_ts($chan) [unixtime]
	news_expire $chan
	set nr [news_count $chan]
	# EB: Added 'how to delete' output for users who send news to wrong channel
	if {$hostdcc == "dcc"} {
		putdcc $nickidx "Added to $chan news file, thanks!"
		putdcc $nickidx "To delete your message, use .news $chan erase $nr now!"
	} {
		putnotc $nickidx "Added to $chan news file, thanks!"
		putnotc $nickidx "To delete your message, use /msg $botnick news $chan erase $nr now!"
	}
	return 1
}

# Delete a news item
# EB 4/7: Multiple channels, DCC/IRC in one proc
proc news_del {nickidx hostdcc hand chan arg} {
	global news botnick news_ncount news_ignore_strangers
	set otherargs [lrange $arg 1 end]
	if {[scan $otherargs "%d" which] != 1} {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "Usage: .news erase <nr>"
			putdcc $nickidx "       Removes message <nr> from $chan news file"
		} {
			if {!$news_ignore_strangers || $hand != "*"} {
				putnotc $nickidx "Usage: /msg $botnick news erase <nr>"
				putnotc $nickidx "       Removes message <nr> from $chan news file"
			}
		}
		return 0
	}
	if {[news_count $chan] == 0} {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "There are no news in $chan to delete."
		} {
			putnotc $nickidx "There are no news in $chan to delete."
		}
		return 0
	}
	if {($which < 1) || ($which > [news_count $chan])} {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "The item number must be between 1 and [news_count $chan]"
		} {
			putnotc $nickidx "The item number must be between 1 and [news_count $chan]"
		}
		return 0
	}
	# We will copy the entries from the original file to a temp file, except
	# for the one to be deleted, then rename the temp file to be the original.
	set next 1
	set fd [open [lindex $news($chan) 0] r]
	set newfd [open "[lindex $news($chan) 0]~new" w]
	while {![eof $fd]} {
		set inp [gets $fd]
		if {[eof $fd]} {break}
		if {[string trim $inp " "] == ""} {continue}
		set who [lindex $inp 0]
		if {$next == $which} {
			# Can only delete your own news unless you're a master
			# EB 4/7: Also allows channel masters to remove everything
			if {([string tolower $who] != [string tolower $hand]) && (![matchattr $hand "m"] && ![matchchanattr $hand "m" $chan])} {
				if {$hostdcc == "dcc"} {
					putdcc $nickidx "It is not possible to erase other users news!"
				} {
					putnotc $nickidx "It is not possible to erase other users news!"
				}
				puts $newfd $inp
			} else {
				# delete it -- ie, do not copy it -- just update the counter
				# Do not use 'incr' just in case it = -1
				set news_ncount($chan) [expr [news_count $chan] - 1]
			}
		} else {
			puts $newfd $inp
		}
		incr next
	}
	close $fd
	close $newfd
	exec /bin/mv [lindex $news($chan) 0]~new [lindex $news($chan) 0]
	if {$hostdcc == "dcc"} {
		putdcc $nickidx "Deleted news $which from $chan news file!"
	} {
		putnotc $nickidx "Deleted news $which from $chan news file!"
	}
	news_expire $chan
	return 1
}

# Read the channel news
# EB 4/7: Multiple channels, DCC/IRC in one proc
proc news_read {nickidx hostdcc hand chan arg} {
	global news botnick news_biggest_ts
	set otherargs [lrange $arg 1 end]
	if {[string trim $otherargs] != ""} {
		set targ [scan $otherargs "%s " textarg]
		if {[string compare [string tolower $textarg] "all"] == 0} {
			if {[news_thereisnews $chan]} {
				if {$hostdcc == "dcc"} {
					putdcc $nickidx "News in $chan at [date] ..."
					news_print $hand $nickidx 0 0 $chan
					putdcc $nickidx "--- end ---"
				} {
					putnotc $nickidx "News in $chan at [date] ..."
					news_print $nickidx 0 0 0 $chan
					putnotc $nickidx "--- end ---"
				}
			} {
				if {$hostdcc == "dcc"} {
					putdcc $nickidx "Sorry, there are no news in $chan!"
				} {
					putnotc $nickidx "Sorry, there are no news in $chan!"
				}
			}
			return 1
		}
		set havearg [scan $otherargs "%d" which]
		if {$havearg} {
			if {($which < 1) || ($which > [news_count $chan])} {
				if {$hostdcc == "dcc"} {
					putdcc $nickidx "The item number must be between 1 and [news_count $chan]"
				} {
					putnotc $nickidx "The item number must be between 1 and [news_count $chan]"
				}
				return 0
			} else {
				if {$hostdcc == "dcc"} {
					news_print $hand $nickidx $which 0 $chan
				} {
					news_print $nickidx 0 $which 0 $chan
				}
				return 1
			}
		}
	}
	# By the time we are here, it must have no arguments - display only new ones!
	if {[news_thereisnews $chan]} {
		set user_ts [news_get_ts $hand $chan]
		if {$user_ts >= $news_biggest_ts($chan)} {
			if {$hostdcc == "dcc"} {
				putdcc $nickidx "Sorry, there are no new news in $chan!"
				putdcc $nickidx "To read all news in $chan, use .news read all"
			} {
				putnotc $nickidx "Sorry, there are no new news in $chan!"
				putnotc $nickidx "To read all news in $chan, use /msg $botnick news read all"
			}
		} else {
			if {$hostdcc == "dcc"} {
				putdcc $nickidx "New news for $hand in $chan at [date] ..."
				news_print $hand $nickidx 0 $user_ts $chan
				putdcc $nickidx "--- end ---"
			} {
				if {$hand == "*"} { set nickhand $nickidx } { set nickhand $hand }
				putnotc $nickidx "New news for $nickhand in $chan at [date] ..."
				news_print $nickidx 0 0 $user_ts $chan
				putnotc $nickidx "--- end ---"
			}
			news_set_ts $hand $news_biggest_ts($chan) $chan
		}
	} else {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "Sorry, there are no new news in $chan!"
		} {
			putnotc $nickidx "Sorry, there are no new news in $chan!"
		}
	}
	if {$hostdcc == "dcc"} {
		putdcc $nickidx "For help with other news commands, use .news help"
	} {
		putnotc $nickidx "For help with other news commands, use /msg $botnick news help"
	}
	return 1
}

# Reset the counter associated with the news file.
# EB 4/7: Multiple channels, DCC/IRC in one proc
proc news_reset {nickidx hostdcc hand chan} {
	global news_ncount news_biggest_ts news_ignore_strangers
	if {![matchattr $hand "m"] && ![matchchanattr $hand "m" $chan} {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "You have no permission to reset the $chan news file."
		} {
			if {!$news_ignore_strangers || $hand != "*"} {
				putnotc $nickidx "You have no permission to reset the $chan news file."
			}
		}
		return 0
	}
	# Just reset the counter, don't touch the file
	set news_ncount($chan) -1
	set news_biggest_ts($chan) 0
	if {$hostdcc == "dcc"} {
		putdcc $nickidx "OK, news for $chan reset."
	} {
		putnotc $nickidx "OK, news for $chan reset."
	}
	return 1
}

# Erase all entries in the news file
# EB 4/7: Multiple channels, DCC/IRC in one proc
proc news_clear {nickidx hostdcc hand chan} {
	global news news_ncount news_newnews news_ignore_strangers
	if {![matchattr $hand "m"]} {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "You have no permission to erase the $chan news file."
		} {
			if {!$news_ignore_strangers || $hand != "*"} {
				putnotc $nickidx "You have no permission to erase the $chan news file."
			}
		}
		return 0
	}
	set fd [open [lindex $news($chan) 0] w]
	puts $fd " "
	close $fd
	set news_ncount($chan) -1
	set news_newnews($chan) 0
	if {$hostdcc == "dcc"} {
		putdcc $nickidx "Okay, news file for $chan cleared!"
	} {
		putnotc $nickidx "Okay, news file for $chan cleared!"
	}
	return 1
}

# Check what news there is to be read
# EB 4/7: Multiple channels, DCC/IRC in one proc
proc news_check {nickidx hostdcc hand chan join} {
	global botnick news_ignore_strangers
	set unread [news_unread $hand $chan]
	if {$unread} {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "There are unread news in $chan ($unread unread, [news_count $chan] total). .news read"
		} {
			putnotc $nickidx "There are unread news in $chan ($unread unread, [news_count $chan] total). /msg $botnick news read"
		}
	} {
		if {!$join} {
			if {$hostdcc == "dcc"} {
				putdcc $nickidx "There are no news news in $chan."
			} {
				if {!$news_ignore_strangers || $hand != "*"} {
					putnotc $nickidx "There are no news news in $chan."
				}
			}
		}
	}
	return 1
}

# Sets default users news channel to 'chan'. If user is not a valid user
# creates a new entry for him using current user/host ident.
# EB 7/7: added
proc news_default {nickidx hostdcc hand chan} {
	global botnick news
	if {$chan == ""} {
		set defchan [user-get $hand newsdefault]
		if {$defchan == ""} {
			if {$hostdcc == "dcc"} {
				putdcc $nickidx "You have no default news channel. You can set one with .news default #channel"
			} {
				putnotc $nickidx "You have no default news channel. You can set one with /msg $botnick news default #channel"
			}
		} {
			if {$hostdcc == "dcc"} {
				putdcc $nickidx "Your default news channel is $defchan."
				putdcc $nickidx "Use .news default none to erase the default news channel."
			} {
				putnotc $nickidx "Your default news channel is $defchan."
				putnotc $nickidx "Use /msg $botnick news default none to erase the default news channel."
			}
		}
		return 0
	}
	if {[lsearch -exact [string tolower [concat [array names news] "none"]] [string tolower $chan]] == -1} {
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "I have no news for channel $chan."
		} {
			putnotc $nickidx "I have no news for channel $chan."
		}
		return 0
	}
	if {$hand == "*"} {
		set uhost [newsmaskhost $nickidx!$hostdcc]
		if {![adduser $nickidx $uhost]} {
			putnotc $nickidx "Could not add you as an user. Try to use another nickname."
			return 0
		} {
			setcomment $nickidx "[ctime [unixtime]] Added for news default"
			set hand $nickidx
		}
	}
	if {$chan == "none"} {
		user-del $hand newsdefault
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "Default news channel erased!"
		} {
			putnotc $nickidx "Default news channel erased!"
		}
	} {
		user-set $hand newsdefault $chan
		if {$hostdcc == "dcc"} {
			putdcc $nickidx "Default news channel set to $chan!"
		} {
			putnotc $nickidx "Default news channel set to $chan!"
		}
	}
	return 1
}

# Expire old news according to date and number of messages in news file
# EB 9/7: added
proc news_expire { chan } {
	global news botnick news_ncount
	if {[lindex $news($chan) 4] == 0} {
		set oldsecs 0
	} {
		set oldsecs [expr [unixtime] - ([lindex $news($chan) 4] * 86400)]
	}

	# Two step expiration: 1) read and copy only news "in time" to a new file
	# 2) read this new file and copy only "max number" news.

	set news_ncount($chan) 0
	set fd [open [lindex $news($chan) 0] r]
	set newfd [open "[lindex $news($chan) 0]~new" w]
	while {![eof $fd] || $oldsecs != 0} {
		set inp [gets $fd]
		if {[eof $fd]} {break}
		if {[string trim $inp " "] == ""} {continue}
		set time [lindex $inp 1]
		if {$time > $oldsecs} {
			incr news_ncount($chan)
			puts $newfd $inp
			# else, delete it -- ie, do not copy it
		}
	}
	close $fd
	close $newfd

	set maxnews [lindex $news($chan) 5]
	set skip [expr $news_ncount($chan) - [lindex $news($chan) 5]]
	set next 0
	set fd [open [lindex $news($chan) 0]~new r]
	set newfd [open "[lindex $news($chan) 0]" w]
	set news_ncount($chan) 0
	while {![eof $fd]} {
		set inp [gets $fd]
		incr next
		if {[eof $fd]} {break}
		if {[string trim $inp " "] == ""} {continue}
		if {$next <= $skip} {continue}
		incr news_ncount($chan)
		puts $newfd $inp
	}
	close $fd
	close $newfd
	exec /bin/rm [lindex $news($chan) 0]~new
	return 0
}

# When joining the channel or partyline, check for new news
# EB 4/7: added
bind join - * join:news_check
bind chon - * chon:news_check

proc join:news_check { nick uhost hand chan } {
	global news botnick
	set chan [string tolower $chan]
	if {$hand != "*"} {
		setlaston $hand $chan
	}
	if {![info exist news($chan)]} { return 0 }
	# Don't annoy other bots
	if {[matchattr $hand b]} { return 0 }
	if {(![lindex $news($chan) 1] && $hand == "*") || $nick == $botnick} {
		return 0
	}
	if {[lindex $news($chan) 2]} {
		news_check $nick $uhost $hand $chan	1
	}
}

proc chon:news_check { hand idx } {
	global news
	set chan [string tolower [lindex [console $idx] 0]]
	if {$chan == "*"} { return 0 }
	if {[info exist news($chan)] && [lindex $news($chan) 2]} {
		news_check $idx "dcc" $hand $chan 1
	}
}

# Display some help
proc news_help {nickidx hostdcc hand chan} {
	if {$hostdcc == "dcc"} {
		putdcc $nickidx "Usage: .news \[#channel\] add <your message>"
		putdcc $nickidx "       .news \[#channel\] erase <nr>"
		putdcc $nickidx "       .news \[#channel\] read \[#\]"
		putdcc $nickidx "       .news \[#channel\] read all"
		putdcc $nickidx "       .news \[#channel\] check"
		if {[matchattr $hand "m"] || [matchchanattr $hand "m" $chan]} {
		putdcc $nickidx "       .news \[#channel\] reset \[#channel\]"
		putdcc $nickidx "       .news \[#channel\] clear \[#channel\]" }
		putdcc $nickidx "       .news default #channel"
	} {
		putnotc $nickidx "/msg $botnick news \[#channel\] <command>"
		putnotc $nickidx "<command> may be ADD, ERASE <nr>, CHECK, READ, READ ALL"
		if {[matchattr $hand "m"] || [matchchanattr $hand "m" $chan]} {
		putnotc $nickidx "                 RESET, CLEAR" }
		putnotc $nickidx "/msg $botnick news default #channel to set your default news channel."
	}
	return 0
}

# EB 4/7: added
proc get_news_chan {nick hand idx arg} {
	global news
	set defchan [user-get $hand newsdefault]
	if {[string index [lindex $arg 0] 0] == "#"} {
		# User has specified a channel
		set chan [lindex $arg 0]
	} elseif {$defchan != ""} {
		# User has a default news channel...
		set chan $defchan
	} {
		if {$idx == 0} {
			# Find channel user joined last
			set high 0
			foreach newschan [array names news] {
				if {[getchanjoin $hand $newschan] > $high} {
					set high [getlaston $hand $newschan]
					set chan $newschan
				}
			}
			if {![info exist chan] && $nick != ""} {
				foreach onchan [array names news] {
					if {[onchan $nick $onchan]} {
						set chan $onchan
						break
					}
				}
			}
			# Last resort, use first configured news channel
			if {![info exist chan]} {
				set chan [lindex [array names $news] 0]
			}
		} {
			set conschan [lindex [console $idx] 0]
			if {$conschan == "*"} {
				set chan ""
			} {
				set chan $conschan
			}
		}
	}
	return $chan
}

bind msg - news msg_news

# The main binding - does the dispatching of requests
# EB 4/7: changed a lot: this is not so simple anymore
proc msg_news {nick uhost hand arg} {
	global news news_ignore_strangers
	if {[string tolower [lindex $arg 0]] == "default"} {
		set chan [string tolower [lindex $arg 1]]
		return [news_default $nick $uhost $hand $chan]
	}
	set chan [get_news_chan $nick $hand 0 $arg]
	if {[string index [lindex $arg 0] 0] == "#"} {
		# Remove channel from arg
		set arg [lreplace $arg 0 0]
	}
	if {$chan == ""} {
		if {!$news_ignore_strangers || $hand != "*"} {
			putnotc $nick "I have no news at all!"
		}
	}
	if {[lsearch -exact [string tolower [array names news]] [string tolower $chan]] == -1} {
		if {!$news_ignore_strangers || $hand != "*"} {
			putnotc $nick "I do not monitor news for channel $chan."
		}
		return 0
	}
	switch [string tolower [lindex $arg 0]] {
		"add"	{
					set pass 0
					if {[lindex [lindex $news($chan) 3] 0] == "all"} {
						set pass 1
					} elseif {[lindex [lindex $news($chan) 3] 0] == "users" && $hand != "*"} {
						set pass 1
					} elseif {[lindex [lindex $news($chan) 3] 0] == "flag"} {
						if {[matchattr $hand [lindex [lindex $news($chan) 3] 1]] || \
						    [matchchanattr $hand [lindex [lindex $news($chan) 3] 1] $chan]} {
							set pass 1
						}
					}
					if {$pass} {
						set r [news_add $nick $uhost $hand $chan $arg]
					} {
						if {$hand != "*" || !$news_ignore_strangers} {
							putnotc $nick "You cannot add news to $chan news file, sorry!"
						}
						set r 0
					}
				}
		"del"	{set r [news_del $nick $uhost $hand $chan $arg]}
		"delete" {set r [news_del $nick $uhost $hand $chan $arg]}
		"erase"	{set r [news_del $nick $uhost $hand $chan $arg]}
		"read"	{
					if {![lindex $news($chan) 1] || $hand != "*"} {
						set r [news_read $nick $uhost $hand $chan $arg]
					} {
						if {$hand != "*" || !$news_ignore_strangers} {
							putnotc $nick "You are not allowed to read $chan news, sorry!"
						}
						set r 0
					}
				}
		"check"	{set r [news_check $nick $uhost $hand $chan 0]}
		"reset"	{set r [news_reset $nick $uhost $hand $chan]}
		"clear"	{set r [news_clear $nick $uhost $hand $chan]}
		default	{set r [news_help $nick $uhost $hand $chan]}
	}
	return $r
}

bind dcc - news dcc_news

# main dcc binding
proc dcc_news {hand idx arg} {
	global news
	if {[string tolower [lindex $arg 0]] == "default"} {
		set chan [string tolower [lindex $arg 1]]
		return [news_default $idx "dcc" $hand $chan]
	}
	set chan [get_news_chan "" $hand $idx $arg]
	if {[string index [lindex $arg 0] 0] == "#"} {
		# Remove channel from arg
		set arg [lreplace $arg 0 0]
	}
	if {[lsearch -exact [string tolower [array names news]] [string tolower $chan]] == -1} {
		putdcc $idx "I have no news for channel $chan."
		return 0
	}
	switch [string tolower [lindex $arg 0]] {
		"add"	{
					set pass 0
					if {[lindex [lindex $news($chan) 3] 0] == "flag"} {
						if {[matchattr $hand [lindex [lindex $news($chan) 3] 1]] || \
						    [matchchanattr $hand [lindex [lindex $news($chan) 3] 1] $chan]} {
							set pass 1
						}
					} { set pass 1 }
					if {$pass} {
						set r [news_add $idx dcc $hand $chan $arg]
					} {
						putdcc $idx "You cannot add news to $chan news file, sorry!"
						set r 0
					}
				}
		"del"	{set r [news_del $idx dcc $hand $chan $arg]}
		"delete" {set r [news_del $idx dcc $hand $chan $arg]}
		"erase"	{set r [news_del $idx dcc $hand $chan $arg]}
		"read"	{set r [news_read $idx dcc $hand $chan $arg]}
		"check"	{set r [news_check $idx dcc $hand $chan 0]}
		"reset"	{set r [news_reset $idx dcc $hand $chan]}
		"clear"	{set r [news_clear $idx dcc $hand $chan]}
		default	{set r [news_help $idx dcc $hand $chan]}
	}
	return $r
}
